import OpenGL.*;

class GU {

    // Pass vertex 0, 1, 2, and will calculate and set the normal
    static private float[] va = new float[3];
    static private float[] vb = new float[3];
    static private float[] vc = new float[3];
    static public void normal(OpenGLwidget gl,float[] v0, float[] v1, float[] v2){
        for(int i = 0; i < 3; i++){
            va[i] = v0[i] - v1[i];
            vb[i] = v1[i] - v2[i];
        }
        normalcp(va, vb, vc);
        gl.normal(vc[0],vc[1],vc[2]);
    }
    static public void normalcp(float[] v1, float[] v2, float[] v){
        v[0] = v1[1]*v2[2] - v1[2]*v2[1];
        v[1] = v1[2]*v2[0] - v1[0]*v2[2];
        v[2] = v1[0]*v2[1] - v1[1]*v2[0];
        float d = (float) Math.sqrt(v[0]*v[0]+v[1]*v[1]+v[2]*v[2]);
        if ( d == 0 ) {
            throw new RuntimeException("Zero length normal");
        }
        v[0] /= d;
        v[1] /= d;
        v[2] /= d;
   }
   static public void drawQuadMesh(OpenGLwidget gl, int[][] qindicies,
      float[][] vdata ){
        gl.pushMatrix();
        gl.begin(gl.QUADS);
        int qLength = qindicies.length;
        for(int i = 0; i < qLength; i++){
            int[] indices = qindicies[i];
            float[] a = vdata[indices[0]];
            float[] b = vdata[indices[1]];
            float[] c = vdata[indices[2]];
            float[] d = vdata[indices[3]];
            normal(gl, a, b, c);
            float texScale = 1.f;
            gl.texCoord(0.f, 0.f);
            gl.vertex(a);
            gl.texCoord(0.f, texScale);
            gl.vertex(b);
            gl.texCoord(texScale, texScale);
            gl.vertex(c);
            gl.texCoord(texScale, 0.f);
            gl.vertex(d);
        }
        gl.end();
        gl.popMatrix();
    }
   static public void drawTriMesh(OpenGLwidget gl, int[][] tindicies,
      float[][] vdata ){
        gl.pushMatrix();
        gl.begin(gl.TRIANGLES);
        int qLength = tindicies.length;
        for(int i = 0; i < qLength; i++){
            int[] indices = tindicies[i];
            float[] a = vdata[indices[0]];
            float[] b = vdata[indices[1]];
            float[] c = vdata[indices[2]];
            normal(gl, a, b, c);
            double texScale = 1.;
            gl.texCoord(0., 0.);
            gl.vertex(a);
            gl.texCoord(0., texScale);
            gl.vertex(b);
            gl.texCoord(texScale, texScale);
            gl.vertex(c);
        }
        gl.end();
        gl.popMatrix();
    }

    /** Assumes 4 component 2D texture.
    */

    static private byte[] downSample(int si, int sj, byte[] source){
	    int di = si / 2;
	    int dj = sj / 2;
	    int sizeInBytes = si * sj * 4;
	    byte[] dest = new byte[di*dj*4];
    	for(int i = 0; i < di; i++){
		    for(int j = 0; j < dj; j++){
			    for(int k = 0; k < 4; k++){
				    int dd = (i*dj+j)*4+k;
				    int a = 0;
				    for (int l = 0; l < 2; l++){
					    for(int m = 0; m < 2; m++){
						    int index = (sizeInBytes + ((i*2+l)*sj + j*2 + m)*4 + k)
							    % sizeInBytes;
						    a += 0xff & source[index];
					    }
				    }
  				  dest[dd] = (byte) (a >> 2);
  			  }
  		  }
  	  }
  	  return dest;
    }

    private static int hackID = 1;
    /** For lame OpenGL implementations that don't implement genTextures
     */

    public static void genTextures(OpenGLwidget gl, int[] ids){
      int length = ids.length;
      gl.genTextures(length, ids);
      if(ids[0] < 0 || ids[0] > (1<<15) ) {
        // Assume a bad implementation
        for(int i = 0; i < length; i++ ){
          ids[i] = hackID++;
        }
      }
    }
    /** For lame OpenGL implementations that don't implement
     * gluMakeMipMap correctly.
     */
    public static void build2DMipmaps(OpenGLwidget gl, int mode, int components,
      int si, int sj, int format, int componentType, byte[] source, int offset){
      if ( mode != gl.TEXTURE_2D || components != 4 ||
        si != 128 || sj != 128
        || format != gl.RGBA || componentType != gl.GL_UNSIGNED_BYTE
        || offset != 0) {
        throw new RuntimeException("Only implemented for 128 x 128 textures.");
      }
      gl.pixelStore(gl.UNPACK_ALIGNMENT, 1);
      byte[] currentMap = source;
      int level = 0;
    	for(int s = 128; s > 0; s = s >> 1){
        gl.texImage2D(mode, level++, components, s, s, 0,
          format, componentType, currentMap, 0);
    	  if ( s > 1 ){
    	    currentMap = downSample(s,s, currentMap);
    	  }
      }
    }

}
