/*
 * Leo Chan -- 1995
 *
 * This file takes care of the C implementation of opening up an
 * X window, assigning an OpenGL graphics context to it, storing that
 * graphics context in the java class data structure.
 *
 * also contains the use() and swap() functions for double buffering
 */

/* 
 * need to include the JAVA internal header files for macros and function
 * prototypes required to maipulated JAVA data structures and functions
 *
 * StubPreamble.h includes the structure and macro definitions neede to
 * convert JAVA data structures into C data structures.
 *
 */

#include "StubPreamble.h"
#include "javaString.h"			/* for converting JAVA strings to C strings */

/*
 * the next thing to include are special headers that were created by
 * JAVAH.  They include the C structure definitions for the JAVA classes
 */
#include "..\OpenGL\OpenGL_OpenGLwidget.h"

/*--------------------------------------------------------------------------
 * here on in is just regular apple pie C
 */

/*
 * next put any C UNIX specific header files that are necessary to implement
 * this native code
 */			

/* using 3Dfx Interactive Glide API.
 */

#include <GL/gl.h>
/* #include <conio.h> */

/* Poor man's event queue */
char mouseActivity;
char rawKeyMode;
int head;
int tail;
#define MAX_SIZE 32
long keys[MAX_SIZE];

static LONG setKey(HWND hWnd, UINT uMsg, int down, WPARAM wParam, LPARAM lParam){
	LONG lRet = 0;
	LONG result = (((long) wParam) & 0xffffL) | (((long) lParam) & 0xffff0000L) ;
	if ( !down ) {
		result |= (1L << 31L);
	}
	if ( rawKeyMode ) {
			keys[head++] = result;
			if ( head >= MAX_SIZE ) head = 0;
		}
		else {
			lRet = DefWindowProc (hWnd, uMsg, wParam, lParam);
		}
	return lRet;
}
long OpenGL_OpenGLwidget_getRawKeyMode(struct HOpenGL_OpenGLwidget *this )
{
	return rawKeyMode != 0;
}
void OpenGL_OpenGLwidget_setRawKeyMode(struct HOpenGL_OpenGLwidget *this, long mode )
{
	rawKeyMode = mode != 0;
	head = tail = 0;
}

long OpenGL_OpenGLwidget_kbhit(struct HOpenGL_OpenGLwidget *this )
{
    return tail != head; /* kbhit() != 0; */
}

long OpenGL_OpenGLwidget_mouseActivity(struct HOpenGL_OpenGLwidget *this )
{
    long result = mouseActivity != 0;
	mouseActivity = 0;
	return result;
}

long OpenGL_OpenGLwidget_getch(struct HOpenGL_OpenGLwidget *this )
{
	long result = 0;
	if ( tail != head ){
		result = keys[tail];
		tail++;
		if ( tail >= MAX_SIZE ) tail = 0;
	}
	/* fprintf(stderr, "Result = %d\n", result); */
    return result;
}

long OpenGL_OpenGLwidget_bufferNumPending(struct HOpenGL_OpenGLwidget * u){
	return 0; /* grBufferNumPending(); */
}

extern int hack_getActiveWindow();
HWND  ghWnd;
HDC   ghDC;
HGLRC ghRC;
BOOL bSetupPixelFormat(HDC);
LONG WINAPI MainWndProc (HWND, UINT, WPARAM, LPARAM);
#include <windowsx.h>

static void ProcessWaitingMessages(){
	MSG msg;
    while (PeekMessage(&msg, NULL, 0, 0, PM_NOREMOVE) == TRUE) {
        if (GetMessage(&msg, NULL, 0, 0)) {
            TranslateMessage(&msg);
            DispatchMessage(&msg);
        } else {
            break;
        }
    }
}

static void cleanClose(){
    if (ghRC){
	    wglMakeCurrent (NULL, NULL) ; 
        wglDeleteContext(ghRC);
		ghRC = 0;
	}
    if (ghDC){
        ReleaseDC(ghWnd, ghDC);
	    ghDC = 0;
	}
}

static void cleanCreate(){
	    ghDC = GetDC(ghWnd);
        if (!bSetupPixelFormat(ghDC))
            PostQuitMessage (0);

        ghRC = wglCreateContext(ghDC);
		if ( ! ghRC ) {
			fprintf(stderr, "wglCreateContext failed: %d\n",
				GetLastError());
		}
        wglMakeCurrent(ghDC, ghRC);
}

int hack_getActiveWindow2(int width, int height){
    WNDCLASS wndclass;

	HWND a = (HWND) hack_getActiveWindow();
	HINSTANCE hInstance = GetWindowInstance(a);

	char* szAppName = "JavaOpenGL";
	/* *((char*) -1) = *((char*) 0); */

	if ( a ) {
		fprintf(stderr,"Hey, a window already exists.\n");
	}

    /* Register the frame class */
    wndclass.style         = 0;
    wndclass.lpfnWndProc   = (WNDPROC)MainWndProc;
    wndclass.cbClsExtra    = 0;
    wndclass.cbWndExtra    = 0;
    wndclass.hInstance     = hInstance;
    wndclass.hIcon         = LoadIcon (hInstance, szAppName);
    wndclass.hCursor       = LoadCursor (NULL,IDC_ARROW);
    wndclass.hbrBackground = (HBRUSH)(COLOR_WINDOW+1);
    wndclass.lpszMenuName  = szAppName;
    wndclass.lpszClassName = szAppName;

    if (!RegisterClass(&wndclass))
        return FALSE;

	/* fprintf(stderr, "%lx", hInstance); */
    /* Create the frame */
    /* Use the WS_POPUP window style to bring up a window that is
     * exactly WIDTHxHEIGHT.
     */
    ghWnd = CreateWindow(szAppName, "OpenGL Demo", 
			 WS_TILEDWINDOW /* WS_POPUP | WS_CLIPSIBLINGS | WS_CLIPCHILDREN */ ,
			 10, 10, width, height,
			 NULL, NULL, hInstance, NULL);

    /* Make sure window was created */
    if (!ghWnd)
        return FALSE;

	ShowWindow(ghWnd, SW_SHOWDEFAULT);
	{
		RECT rect;
		int w, h;
		GetClientRect(ghWnd, &rect);
		w = rect.right - rect.left;
		h = rect.bottom - rect.top;
		SetWindowPos(ghWnd, HWND_TOP, 10, 10, width * 2 - w, height * 2 - h,
			0);
	}
    UpdateWindow(ghWnd);
	return TRUE;
}

LONG WINAPI MainWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
    LONG lRet = 1;
    PAINTSTRUCT ps;
    RECT rect;
		if ( ghWnd != hWnd) {
			// fprintf(stderr, "%lx != %lx\n", ghWnd, hWnd);
			ghWnd = hWnd;
		}

    switch (uMsg) {
    case WM_CREATE:
		cleanCreate();
        break;

    case WM_PAINT:
        BeginPaint(hWnd, &ps);
        EndPaint(hWnd, &ps);
        break;

    case WM_SIZE:
		cleanClose();
		cleanCreate();
        GetClientRect(hWnd, &rect);
        /* resize(rect.right, rect.bottom); */
        break;

    case WM_CLOSE:
		cleanClose();

        DestroyWindow (hWnd);
        break;

    case WM_DESTROY:
        cleanClose();

        PostQuitMessage (0);
        break;

	case WM_MOUSEMOVE:
	case WM_LBUTTONDOWN:
	case WM_MBUTTONDOWN:
	case WM_RBUTTONDOWN:
		mouseActivity = 1;
		break;

	case WM_KEYDOWN:
		lRet = setKey(hWnd, uMsg, TRUE, wParam, lParam);
		break;
	case WM_KEYUP:
		fprintf(stderr, "WM_KEYUP\n");
		lRet = setKey(hWnd, uMsg, FALSE, wParam, lParam);
		break;

	case WM_CHAR:
		if ( ! rawKeyMode ) {
			keys[head++] = (char) wParam;
			if ( head >= MAX_SIZE ) head = 0;
		}
		else {
			lRet = DefWindowProc (hWnd, uMsg, wParam, lParam);
		}
		break;

    default:
        lRet = DefWindowProc (hWnd, uMsg, wParam, lParam);
        break;
    }

    return lRet;
}

/*boolean*/ long OpenGL_OpenGLwidget_openOpenGLwidget(struct HOpenGL_OpenGLwidget * u,long w,long h){
        return hack_getActiveWindow2(w, h);
}

BOOL bSetupPixelFormat(HDC hdc)
{
    PIXELFORMATDESCRIPTOR pfd, *ppfd;
    int pixelformat;

    ppfd = &pfd;

    ppfd->nSize = sizeof(PIXELFORMATDESCRIPTOR);
    ppfd->nVersion = 1;
    ppfd->dwFlags = PFD_SUPPORT_OPENGL | PFD_DOUBLEBUFFER;
    ppfd->dwLayerMask = PFD_MAIN_PLANE;
    ppfd->iPixelType = PFD_TYPE_RGBA;
    ppfd->cColorBits = 16;
    ppfd->cDepthBits = 16;
    ppfd->cAccumBits = 0;
    ppfd->cStencilBits = 0;

    pixelformat = ChoosePixelFormat(hdc, ppfd);

    if ((pixelformat = ChoosePixelFormat(hdc, ppfd)) == 0) {
        MessageBox(NULL, "ChoosePixelFormat failed", "Error", MB_OK);
        return FALSE;
    }

    if (pfd.dwFlags & PFD_NEED_PALETTE) {
        MessageBox(NULL, "Needs palette", "Error", MB_OK);
        return FALSE;
    }

    if (SetPixelFormat(hdc, pixelformat, ppfd) == FALSE) {
        MessageBox(NULL, "SetPixelFormat failed", "Error", MB_OK);
        return FALSE;
    }

    return TRUE;
}


/*boolean*/ long OpenGL_OpenGLwidget_use( struct HOpenGL_OpenGLwidget *this )
{
    if ( ghDC && ghRC ) {
		wglMakeCurrent(ghDC, ghRC);
	}
	return 1;
}

/*boolean*/ long OpenGL_OpenGLwidget_swap2( struct HOpenGL_OpenGLwidget *this )
{
	BOOL result = 0;
	ProcessWaitingMessages();
	if ( ghDC ) {
		result = SwapBuffers(ghDC);
	}
	if ( ! result ) {
		return 0;
	}
	return 1;
}

void OpenGL_OpenGLwidget_shutdown( struct HOpenGL_OpenGLwidget *this )
{
        cleanClose();
}
