import OpenGL.*;

class Torus implements Drawable {
  private int base;
  public Torus(OpenGLwidget gl, double innerRadius, double outerRadius){
    this(gl, innerRadius, outerRadius, 20, 10);
  }
  public Torus(OpenGLwidget gl, double innerRadius2, double outerRadius2,
    int rings, int sides)
  {
      int i, j;
      float theta1, phi1, theta2, phi2;
      float v0[] = new float[3];
      float v1[] = new float[3];
      float v2[] = new float[3];
      float v3[] = new float[3];
      float n0[] = new float[3];
      float n1[] = new float[3];
      float n2[] = new float[3];
      float n3[] = new float[3];

      float innerRadius = (float) innerRadius2;
      float outerRadius = (float) outerRadius2;
      base = gl.genLists(1);

      gl.newList(base, gl.COMPILE);
      for (i = 0; i < rings; i++) {
  	    theta1 = (float)((float)i * 2.0 * Math.PI / rings);
  	    theta2 = (float)((float)(i + 1) * 2.0 * Math.PI / rings);
  	    for (j = 0; j < sides; j++) {
  	      phi1 = (float)((float)j * 2.0 * Math.PI / sides);
  	      phi2 = (float)((float)(j + 1) * 2.0 * Math.PI / sides);

  	      v0[0] = (float)(Math.cos(theta1) * (outerRadius + innerRadius * Math.cos(phi1)));
  	    v0[1] = (float)(-Math.sin(theta1) * (outerRadius + innerRadius * Math.cos(phi1)));
  	    v0[2] = (float)(innerRadius * Math.sin(phi1));

  	    v1[0] = (float)(Math.cos(theta2) * (outerRadius + innerRadius * Math.cos(phi1)));
  	    v1[1] = (float)(-Math.sin(theta2) * (outerRadius + innerRadius * Math.cos(phi1)));
  	    v1[2] = (float)(innerRadius * Math.sin(phi1));

  	    v2[0] = (float)(Math.cos(theta2) * (outerRadius + innerRadius * Math.cos(phi2)));
  	    v2[1] = (float)(-Math.sin(theta2) * (outerRadius + innerRadius * Math.cos(phi2)));
  	    v2[2] = (float)(innerRadius * Math.sin(phi2));

  	    v3[0] = (float)(Math.cos(theta1) * (outerRadius + innerRadius * Math.cos(phi2)));
  	    v3[1] = (float)(-Math.sin(theta1) * (outerRadius + innerRadius * Math.cos(phi2)));
  	    v3[2] = (float)(innerRadius * Math.sin(phi2));

  	    n0[0] = (float)(Math.cos(theta1) * (Math.cos(phi1)));
  	    n0[1] = (float)(-Math.sin(theta1) * (Math.cos(phi1)));
  	    n0[2] = (float)(Math.sin(phi1));

  	    n1[0] = (float)(Math.cos(theta2) * (Math.cos(phi1)));
  	    n1[1] = (float)(-Math.sin(theta2) * (Math.cos(phi1)));
  	    n1[2] = (float)(Math.sin(phi1));

  	    n2[0] = (float)(Math.cos(theta2) * (Math.cos(phi2)));
  	    n2[1] = (float)(-Math.sin(theta2) * (Math.cos(phi2)));
  	    n2[2] = (float)(Math.sin(phi2));

  	    n3[0] = (float)(Math.cos(theta1) * (Math.cos(phi2)));
  	    n3[1] = (float)(-Math.sin(theta1) * (Math.cos(phi2)));
  	    n3[2] = (float)(Math.sin(phi2));

  	      gl.begin(gl.QUADS);
  	        //gl.texCoord(v3[0], v3[1]);
      		  gl.normal(n3);
      		  gl.vertex(v3);
  	        //gl.texCoord(v2[0], v2[1]);
      		  gl.normal(n2);
      		  gl.vertex(v2);
  	        //gl.texCoord(v1[0], v1[1]);
      		  gl.normal(n1);
      		  gl.vertex(v1);
  	        //gl.texCoord(v0[0], v0[1]);
      		  gl.normal(n0);
            gl.vertex(v0);
  	      gl.end();
  	    }
      }
      gl.endList();
  }
    public void draw(OpenGLwidget gl){
      gl.callList(base);
    }
}